from luma.metadata.sources.powerbi.extract import powerbi
from luma.metadata.sources.powerbi.models import WorkspaceInfo
from luma.metadata.sources.powerbi.transform import transform


def pipeline():
    source = powerbi()
    metadata = WorkspaceInfo(**next(iter(source)))
    yield transform(metadata)


if __name__ == "__main__":
    manifest = next(iter(pipeline()))
    # print(manifest)



import time
from enum import Enum
from pathlib import Path
from typing import Optional

import typer
from rich import print
from rich.console import Console
from rich.panel import Panel

from lumaCLI.models import RequestInfo
from lumaCLI.models.dbt import ManifestDict
from lumaCLI.utils import (
    check_ingestion_results,
    check_ingestion_status,
    json_to_dict,
    perform_ingestion_request,
)
from lumaCLI.utils.options import (
    DryRun,
    Follow,
    LumaURL,
    MetadataDir,
)

# Create Typer application
app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)
console = Console()


class IngestionStatus(Enum):
    successful = 0
    failed = 1
    pending = 2


@app.command()
def ingest(
    metadata_dir: Path = MetadataDir,
    luma_url: str = LumaURL,
    dry_run: bool = DryRun,
    follow: bool = Follow,
):
    """
    Ingests a bundle of JSON files (manifest.json, catalog.json, sources.json, run_results.json)
    located in the specified directory to a Luma endpoint.
    manifest.json and catalog.json are required, if not present, the command will fail.
    Uses the current working directory if 'metadata_dir' is not specified.
    """
    # Define JSON paths
    manifest_json_path = metadata_dir / "manifest.json"
    catalog_json_path = metadata_dir / "catalog.json"
    sources_json_path = metadata_dir / "sources.json"
    run_results_json_path = metadata_dir / "run_results.json"

    # Ensure all JSON files are valid
    if not manifest_json_path.is_file():
        print(Panel(f"[red]{manifest_json_path.absolute()} is not a file[/red]"))
        raise typer.Exit(1)
    if not catalog_json_path.is_file():
        print(Panel(f"[red]{catalog_json_path.absolute()} is not a file[/red]"))
        raise typer.Exit(1)

    # Convert each JSON to dict
    manifest_dict: Optional[dict] = json_to_dict(json_path=manifest_json_path)
    catalog_dict: Optional[dict] = json_to_dict(json_path=catalog_json_path)
    sources_dict: Optional[dict] = json_to_dict(json_path=sources_json_path)
    run_results_dict: Optional[dict] = json_to_dict(json_path=run_results_json_path)

    # Validate manifest_dict using ManifestDict
    ManifestDict.validate(manifest_dict)

    # Define bundle dict
    bundle_dict = {
        "manifest_json": manifest_dict,
        "catalog_json": catalog_dict,
        "sources_json": sources_dict,
        "run_results_json": run_results_dict,
    }

    # If in dry run mode, print the bundle and exit
    if dry_run:
        print(bundle_dict)
        raise typer.Exit(0)

    # Create the request information
    endpoint = f"{luma_url}/api/v1/dbt/"
    request_info = RequestInfo(
        url=endpoint,
        method="POST",
        payload=bundle_dict,
        verify=False,
    )

    response, ingestion_uuid = perform_ingestion_request(request_info)
    if not response.ok:
        raise typer.Exit(1)

    if follow and ingestion_uuid:
        ingestion_status = None

        with console.status("Waiting...", spinner="dots") as spinner:
            for _ in range(30):
                ingestion_status = check_ingestion_status(luma_url, ingestion_uuid)
                if ingestion_status == IngestionStatus.successful.value:
                    response = check_ingestion_results(luma_url, ingestion_uuid)
                    print()
                    print(f"Ingestion results for ID {ingestion_uuid}:")
                    print()
                    print(response)
                    return

                if ingestion_status == IngestionStatus.failed.value:
                    print()
                    print(f"Ingestion failed for ID {ingestion_uuid}")
                    return

                if ingestion_status == IngestionStatus.pending.value:
                    time.sleep(1)

        if ingestion_status != IngestionStatus.successful.value:
            print(
                f"Ingestion did not complete successfully within the wait period. Status: {ingestion_status}"
            )


@app.command()
def send_test_results(
    metadata_dir: Path = MetadataDir,
    luma_url: str = LumaURL,
    dry_run: bool = DryRun,
    follow: bool = Follow,
):
    """
    Sends the 'run_results.json' file located in the specified directory to a Luma endpoint.
    The command will fail if the 'run_results.json' file is not present in the directory.
    The current working directory is used if 'metadata_dir' is not specified.
    """

    # Define the path to 'run_results.json'
    run_results_path = Path(metadata_dir) / "run_results.json"

    # Convert 'run_results.json' to dict
    run_results_dict = json_to_dict(json_path=run_results_path)

    # If in dry run mode, print the test results and exit
    if dry_run:
        print(run_results_dict)
        raise typer.Exit(0)

    # Create and return the request information for test results
    endpoint = f"{luma_url}/api/v1/dbt/run_results/"
    request_info = RequestInfo(
        url=endpoint,
        method="POST",
        payload=run_results_dict,
        verify=False,
    )

    response, ingestion_uuid = perform_ingestion_request(request_info)
    if not response.ok:
        raise typer.Exit(1)

    if follow and ingestion_uuid:
        ingestion_status = None

        with console.status("Waiting...", spinner="dots") as spinner:
            for _ in range(30):
                ingestion_status = check_ingestion_status(luma_url, ingestion_uuid)
                if ingestion_status == IngestionStatus.successful.value:
                    response = check_ingestion_results(luma_url, ingestion_uuid)
                    print()
                    print(f"Ingestion results for ID {ingestion_uuid}:")
                    print()
                    print(response)
                    return

                if ingestion_status == IngestionStatus.failed.value:
                    print()
                    print(f"Ingestion failed for ID {ingestion_uuid}")
                    return

                if ingestion_status == IngestionStatus.pending.value:
                    time.sleep(1)

        if ingestion_status != IngestionStatus.successful.value:
            print(
                f"Ingestion did not complete successfully within the wait period. Status: {ingestion_status}"
            )


# Run the application
if __name__ == "__main__":
    app()



from typing import List, Optional

from pydantic import BaseModel, EmailStr, root_validator, validator


allowed_icons = (
    "AcademicCap",
    "AdjustmentsHorizontal",
    "AdjustmentsVertical",
    "ArchiveBox",
    "ArchiveBoxArrowDown",
    "ArchiveBoxXMark",
    "ArrowDown",
    "ArrowDownCircle",
    "ArrowDownLeft",
    "ArrowDownOnSquare",
    "ArrowDownOnSquareStack",
    "ArrowDownRight",
    "ArrowDownTray",
    "ArrowLeft",
    "ArrowLeftCircle",
    "ArrowLeftOnRectangle",
    "ArrowLongDown",
    "ArrowLongLeft",
    "ArrowLongRight",
    "ArrowLongUp",
    "ArrowPath",
    "ArrowPathRoundedSquare",
    "ArrowRight",
    "ArrowRightCircle",
    "ArrowRightOnRectangle",
    "ArrowSmallDown",
    "ArrowSmallLeft",
    "ArrowSmallRight",
    "ArrowSmallUp",
    "ArrowTopRightOnSquare",
    "ArrowTrendingDown",
    "ArrowTrendingUp",
    "ArrowUp",
    "ArrowUpCircle",
    "ArrowUpLeft",
    "ArrowUpOnSquare",
    "ArrowUpOnSquareStack",
    "ArrowUpRight",
    "ArrowUpTray",
    "ArrowUturnDown",
    "ArrowUturnLeft",
    "ArrowUturnRight",
    "ArrowUturnUp",
    "ArrowsPointingIn",
    "ArrowsPointingOut",
    "ArrowsRightLeft",
    "ArrowsUpDown",
    "AtSymbol",
    "Backspace",
    "Backward",
    "Banknotes",
    "Bars2",
    "Bars3",
    "Bars3BottomLeft",
    "Bars3BottomRight",
    "Bars3CenterLeft",
    "Bars4",
    "BarsArrowDown",
    "BarsArrowUp",
    "Battery0",
    "Battery100",
    "Battery50",
    "Beaker",
    "Bell",
    "BellAlert",
    "BellSlash",
    "BellSnooze",
    "Bolt",
    "BoltSlash",
    "BookOpen",
    "Bookmark",
    "BookmarkSlash",
    "BookmarkSquare",
    "Briefcase",
    "BugAnt",
    "BuildingLibrary",
    "BuildingOffice",
    "BuildingOffice2",
    "BuildingStorefront",
    "Cake",
    "Calculator",
    "Calendar",
    "CalendarDays",
    "Camera",
    "ChartBar",
    "ChartBarSquare",
    "ChartPie",
    "ChatBubbleBottomCenter",
    "ChatBubbleBottomCenterText",
    "ChatBubbleLeft",
    "ChatBubbleLeftEllipsis",
    "ChatBubbleLeftRight",
    "ChatBubbleOvalLeft",
    "ChatBubbleOvalLeftEllipsis",
    "Check",
    "CheckBadge",
    "CheckCircle",
    "ChevronDoubleDown",
    "ChevronDoubleLeft",
    "ChevronDoubleRight",
    "ChevronDoubleUp",
    "ChevronDown",
    "ChevronLeft",
    "ChevronRight",
    "ChevronUp",
    "ChevronUpDown",
    "CircleStack",
    "Clipboard",
    "ClipboardDocument",
    "ClipboardDocumentCheck",
    "ClipboardDocumentList",
    "Clock",
    "Cloud",
    "CloudArrowDown",
    "CloudArrowUp",
    "CodeBracket",
    "CodeBracketSquare",
    "Cog",
    "Cog6Tooth",
    "Cog8Tooth",
    "CommandLine",
    "ComputerDesktop",
    "CpuChip",
    "CreditCard",
    "Cube",
    "CubeTransparent",
    "CurrencyBangladeshi",
    "CurrencyDollar",
    "CurrencyEuro",
    "CurrencyPound",
    "CurrencyRupee",
    "CurrencyYen",
    "CursorArrowRays",
    "CursorArrowRipple",
    "DevicePhoneMobile",
    "DeviceTablet",
    "Document",
    "DocumentArrowDown",
    "DocumentArrowUp",
    "DocumentChartBar",
    "DocumentCheck",
    "DocumentDuplicate",
    "DocumentMagnifyingGlass",
    "DocumentMinus",
    "DocumentPlus",
    "DocumentText",
    "EllipsisHorizontal",
    "EllipsisHorizontalCircle",
    "EllipsisVertical",
    "Envelope",
    "EnvelopeOpen",
    "ExclamationCircle",
    "ExclamationTriangle",
    "Eye",
    "EyeDropper",
    "EyeSlash",
    "FaceFrown",
    "FaceSmile",
    "Film",
    "FingerPrint",
    "Fire",
    "Flag",
    "Folder",
    "FolderArrowDown",
    "FolderMinus",
    "FolderOpen",
    "FolderPlus",
    "Forward",
    "Funnel",
    "Gif",
    "Gift",
    "GiftTop",
    "GlobeAlt",
    "GlobeAmericas",
    "GlobeAsiaAustralia",
    "GlobeEuropeAfrica",
    "HandRaised",
    "HandThumbDown",
    "HandThumbUp",
    "Hashtag",
    "Heart",
    "Home",
    "HomeModern",
    "Identification",
    "Inbox",
    "InboxArrowDown",
    "InboxStack",
    "InformationCircle",
    "Key",
    "Language",
    "Lifebuoy",
    "LightBulb",
    "Link",
    "ListBullet",
    "LockClosed",
    "LockOpen",
    "MagnifyingGlass",
    "MagnifyingGlassCircle",
    "MagnifyingGlassMinus",
    "MagnifyingGlassPlus",
    "Map",
    "MapPin",
    "Megaphone",
    "Microphone",
    "Minus",
    "MinusCircle",
    "MinusSmall",
    "Moon",
    "MusicalNote",
    "Newspaper",
    "NoSymbol",
    "PaintBrush",
    "PaperAirplane",
    "PaperClip",
    "Pause",
    "PauseCircle",
    "Pencil",
    "PencilSquare",
    "Phone",
    "PhoneArrowDownLeft",
    "PhoneArrowUpRight",
    "PhoneXMark",
    "Photo",
    "Play",
    "PlayCircle",
    "PlayPause",
    "Plus",
    "PlusCircle",
    "PlusSmall",
    "Power",
    "PresentationChartBar",
    "PresentationChartLine",
    "Printer",
    "PuzzlePiece",
    "QrCode",
    "QuestionMarkCircle",
    "QueueList",
    "Radio",
    "ReceiptPercent",
    "ReceiptRefund",
    "RectangleGroup",
    "RectangleStack",
    "RocketLaunch",
    "Rss",
    "Scale",
    "Scissors",
    "Server",
    "ServerStack",
    "Share",
    "ShieldCheck",
    "ShieldExclamation",
    "ShoppingBag",
    "ShoppingCart",
    "Signal",
    "SignalSlash",
    "Sparkles",
    "SpeakerWave",
    "SpeakerXMark",
    "Square2Stack",
    "Square3Stack3d",
    "Squares2x2",
    "SquaresPlus",
    "Star",
    "Stop",
    "StopCircle",
    "Sun",
    "Swatch",
    "TableCells",
    "Tag",
    "Ticket",
    "Trash",
    "Trophy",
    "Truck",
    "Tv",
    "User",
    "UserCircle",
    "UserGroup",
    "UserMinus",
    "UserPlus",
    "Users",
    "Variable",
    "VideoCamera",
    "VideoCameraSlash",
    "ViewColumns",
    "ViewfinderCircle",
    "Wallet",
    "Wifi",
    "Window",
    "Wrench",
    "WrenchScrewdriver",
    "XCircle",
    "XMark",
)


class Group(BaseModel):
    """Represents a group with metadata, slug, labels, and an optional icon.

    Attributes:
        meta_key (str): A key for metadata associated with the group.
        slug (str): A slug for the group, used in URLs or as an identifier.
        label_plural (str): The plural label for the group.
        label_singular (str): The singular label for the group.
        icon (Optional[str]): An optional icon for the group. Must be one of the allowed
            icons.
        in_sidebar (bool, optional): Whether the group should be displayed in the
            sidebar.
        visible (bool, optional): Whether the group should be displayed in Luma UI.

    Methods:
        icon_validator: Validates that the provided icon (if any) is in the allowed set.
    """

    meta_key: str
    slug: str
    label_plural: str
    label_singular: str
    icon: Optional[str]
    in_sidebar: Optional[bool] = True
    visible: Optional[bool] = True

    @validator("icon")
    def icon_validator(cls, v):
        """Validates the icon attribute.

        Ensures that if an icon is provided, it is one of the pre-approved icons.

        Args:
            v (str): The icon to validate.

        Returns:
            str: The validated icon.

        Raises:
            ValueError: If the icon is not in the allowed set.
        """
        if v is not None and v not in allowed_icons:
            raise ValueError("Icon is not one of the allowed options")
        return v


class Owner(BaseModel):
    """Represents an owner with email, name, and title.

    Attributes:
        email (EmailStr): The email address of the owner.
        first_name (str): The first name of the owner.
        last_name (str): The last name of the owner.
        title (str): The title of the owner.
    """

    email: EmailStr
    first_name: str
    last_name: str
    title: str


class Config(BaseModel):
    """Configuration model holding information about groups and owners.

    Attributes:
        groups (Optional[List[Group]]): A list of Group objects. Ensures uniqueness of
            meta_keys and slugs among groups.
        owners (Optional[List[Owner]]): A list of Owner objects.

    Methods:
        validate_unique: Validates the uniqueness of meta_keys and slugs within the
            groups.
    """

    groups: Optional[List[Group]]
    owners: Optional[List[Owner]]

    @root_validator
    def validate_unique(cls, values):
        """Validates the uniqueness of 'meta_key' and 'slug' for each group.

        Args:
            values (dict): The values to validate.

        Returns:
            dict: The validated values.

        Raises:
            ValueError: If 'meta_key' or 'slug' is not unique across all groups.
        """
        groups = values.get("groups")
        if groups:
            # Check for unique meta_key
            meta_keys = {group.meta_key for group in groups}
            if len(meta_keys) != len(groups):
                raise ValueError("meta_key must be unique for each group")

            # Check for unique slug
            slugs = {group.slug for group in groups}
            if len(slugs) != len(groups):
                raise ValueError("slug must be unique for each group")
        return values



from lumaCLI.utils.luma_utils import (
    get_config,
    init_config,
    run_command,
    send_config,
    perform_request,
    json_to_dict,
    perform_ingestion_request,
    check_ingestion_results,
    check_ingestion_status,
    print_response,
)
from lumaCLI.utils.postgres_utils import (
    create_conn,
    generate_pg_dump_content,
    get_db_metadata,
    get_pg_dump_tables_info,
    get_pg_dump_views_info,
    get_tables_row_counts,
    get_tables_size_info,
)



"""Ingest PostgreSQL metadata into Luma."""

from pathlib import Path
from typing import Any

from requests import Response
from rich import print
from rich.panel import Panel
import typer

from luma.models import Config
from luma.utils import (
    get_config,
    get_db_metadata,
    perform_ingestion_request,
    send_config,
)
from luma.utils.options import (
    ConfigDir,
    DryRun,
    LumaURL,
    NoConfig,
)


app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)


@app.command()
def ingest(
    luma_url: str = LumaURL,
    username: str = typer.Option(
        ...,
        "--username",
        "-u",
        envvar="LUMA_POSTGRES_USERNAME",
        help="The username for the PostgreSQL database.",
        prompt="PostgreSQL username",
    ),
    database: str = typer.Option(
        ...,
        "--database",
        "-d",
        envvar="LUMA_POSTGRES_DATABASE",
        help="The name of the PostgreSQL database.",
        prompt="PostgreSQL database",
    ),
    host: str = typer.Option(
        "localhost",
        "--host",
        "-h",
        envvar="LUMA_POSTGRES_HOST",
        help="The host address of the PostgreSQL database.",
    ),
    port: str = typer.Option(
        "5432",
        "--port",
        "-p",
        envvar="LUMA_POSTGRES_PORT",
        help="The port number for the PostgreSQL database.",
    ),
    password: str = typer.Option(
        ...,
        "--password",
        "-P",
        envvar="LUMA_POSTGRES_PASSWORD",
        help="The password for the PostgreSQL database.",
        prompt="PostgreSQL password",
        hide_input=True,
    ),
    dry_run: bool = DryRun,
    config_dir: Path = ConfigDir,
    no_config: bool = NoConfig,
) -> Response:
    """Ingest metadata from PostgreSQL database into a Luma ingestion endpoint."""
    should_send_config = not no_config
    config = None

    if should_send_config:
        try:
            config: Config = get_config(config_dir=config_dir)
        except FileNotFoundError:
            print(
                Panel(
                    "[blue]No config files found. Continuing with the operation...[/blue]"
                )
            )

    # Retrieve database metadata.
    db_metadata: dict[str, list[dict[str, Any]]] = get_db_metadata(
        username=username, database=database, host=host, port=port, password=password
    )

    # In dry run mode, print the database metadata and exit.
    if dry_run:
        print(db_metadata)
        raise typer.Exit(0)

    endpoint = f"{luma_url}/api/v1/postgres"

    # Send ingestion request.
    if config and should_send_config:
        send_config(config=config, luma_url=luma_url)

    response = perform_ingestion_request(
        url=endpoint,
        method="POST",
        payload=db_metadata,
        verify=False,
    )
    if not response.ok:
        raise typer.Exit(1)

    return response


if __name__ == "__main__":
    app()



import json
import subprocess
import traceback
from enum import Enum
from pathlib import Path
from typing import Optional, Tuple, Union

import requests
import typer
import yaml
from rich import print
from rich.console import Console
from rich.panel import Panel

from lumaCLI.models import Config, RequestInfo


class IngestionStatus(Enum):
    successful = 0
    failed = 1
    pending = 2


# Create console for rich output
console = Console()

CONFIG_YAML_EXAMPLE = """# Example:
#
# groups:
#   - meta_key: "domain"
#     slug: "domains"
#     label_plural: "Domains"
#     label_singular: "Domain"
#     icon: "Cube"
#     in_sidebar: true
#     visible: true
#   - meta_key: "true_source"
#     slug: "sources"
#     label_plural: "Sources"
#     label_singular: "Source"
#     icon: "Cloud"
#     in_sidebar: true
"""
OWNERS_YAML_EXAMPLE = """# Example:
#
# owners:
#   - email: "some@one.com"
#     first_name: "Dave"
#     last_name: "Smith"
#     title: "Director"
#   - email: "other@person.com"
#     first_name: "Michelle"
#     last_name: "Dunne"
#     title: "CTO"
#   - email: "someone@else.com"
#     first_name: "Dana"
#     last_name: "Pawlak"
#     title: "HR Manager"
"""


def json_to_dict(json_path: str) -> Optional[dict]:
    """Converts a JSON file to a Python dictionary.

    Args:
        json_path (str): The path to the JSON file.

    Returns:
        Optional[dict]: A dictionary representation of the JSON file, or None if an
            error occurs.
    """

    try:
        with open(json_path, "r") as json_file:
            # Load JSON data from file
            json_data: dict = json.load(json_file)
    except:
        json_data = None
    return json_data


def run_command(command: str, capture_output: bool = False) -> Optional[str]:
    """
    Execute a shell command and optionally capture its output.

    Args:
        command (str): The shell command to be executed.
        capture_output (bool, optional): Flag to determine if the command's output
            should be captured. Defaults to False.

    Returns:
        Optional[str]: The standard output of the command if `capture_output` is True,
            otherwise None.

    Raises:
        typer.Exit: Exits the script if the command execution fails.
    """

    try:
        if capture_output:
            result = subprocess.run(
                command, shell=True, check=True, capture_output=True, text=True
            )
            return result.stdout.strip()
        else:
            subprocess.run(command, shell=True, check=True)
            return None
    except subprocess.CalledProcessError as e:
        console.print(
            Panel.fit(
                "[bold red]ERROR[/bold red]: An error occurred while running the command: [bold yellow]{}[/bold yellow]".format(
                    e
                ),
                title="Error",
                border_style="red",
            )
        )
        if e.output:
            console.print("[bold cyan]Output[/bold cyan]: {}".format(e.output))
        if e.stderr:
            console.print("[bold red]Error[/bold red]: {}".format(e.stderr))
    raise typer.Exit(1)


def init_config(config_dir: Union[Path, str] = "./.luma", force: bool = False):
    """
    Initialize configuration files in the specified directory.

    Args:
        config_dir (Union[Path, str], optional): The directory where configuration files
            will be created. Defaults to "./.luma".
        force (bool, optional): If True, existing configuration files will be
            overwritten. Defaults to False.

    Raises:
        FileExistsError: If configuration files already exist and `force` is not set to
            True.
    """
    config_dir = Path(config_dir)

    config_path = config_dir / "config.yaml"
    owners_path = config_dir / "owners.yaml"

    if force:
        config_path.unlink(missing_ok=True)
        owners_path.unlink(missing_ok=True)
        try:
            config_dir.rmdir()
        except FileNotFoundError:
            pass

    if not config_path.exists() and not owners_path.exists():
        config_dir.mkdir(exist_ok=True)
        config_path.touch(exist_ok=False)
        owners_path.touch(exist_ok=False)
    else:
        raise FileExistsError

    config_path.write_text(CONFIG_YAML_EXAMPLE)
    owners_path.write_text(OWNERS_YAML_EXAMPLE)


def get_config(config_dir: Union[Path, str] = "./.luma") -> Optional[Config]:
    """Retrieve configuration data from YAML files in the specified directory.

    Args:
        config_dir (Union[Path, str], optional): The directory containing the
            configuration files. Defaults to "./.luma".

    Returns:
        Optional[Config]: The configuration object if the configuration is successfully
            loaded, otherwise None.

    Raises:
        FileNotFoundError: If the configuration files are missing.
        typer.Abort: If there is an error parsing the YAML files.
    """
    config_dir = Path(config_dir)

    config_path = config_dir / "config.yaml"
    owners_path = config_dir / "owners.yaml"

    config_missing = True
    owners_missing = True

    config_dict = {}
    config_data = {}
    owners_data = {}

    if config_path.exists():
        config_missing = False
        with config_path.open("r") as f:
            try:
                config_data: Optional[dict] = yaml.safe_load(f)
            except yaml.YAMLError as exc:
                print(f"Error parsing YAML file: {exc}")
                raise typer.Abort()

    if owners_path.exists():
        owners_missing = False
        with owners_path.open("r") as f:
            try:
                owners_data: Optional[dict] = yaml.safe_load(f)

            except yaml.YAMLError as exc:
                print(f"Error parsing YAML file: {exc}")
                raise typer.Abort()

    if config_missing and owners_missing:
        raise FileNotFoundError

    if config_data is not None:
        config_dict.update(config_data)

    if owners_data is not None:
        config_dict.update(owners_data)

    return Config(**config_dict)


def print_response(response: requests.Response):
    # PRINT RESPONSE
    if response.ok:
        success_message = "[green]The request was successful!\nResponse:[/green]"
        print(Panel(success_message))
        try:
            print(response.json())
        except:
            print(Panel("[red]Error at printing items ingested[/red]"))
    else:
        try:
            error_message = f"[red]URL: {response.url}[/red]\n[red]An HTTP error occurred, response status code[/red]: {response.status_code} {response.json()['message']}"
        except:
            error_message = f"[red]URL: {response.url}[/red]\n[red]An HTTP error occurred, response status code[/red]: {response.status_code} {response.text}"
        print(Panel(error_message))


def send_config(config: Config, luma_url: str) -> requests.Response:
    """Send configuration data to a specified URL.

    Args:
        config (Config): The configuration data to be sent.
        luma_url (str): The URL where the configuration data will be sent.

    Returns:
        requests.Response: The response from the server after sending the configuration data.

    Raises:
        typer.Exit: If there is an error in sending the configuration data.
    """
    print(Panel(f"[yellow]Sending config info to luma[/yellow]"))

    try:
        response = requests.request(
            method="POST",
            url=f"{luma_url}/api/v1/config",
            json=config.dict(),
            verify=False,
            timeout=(
                21.05,
                60 * 30,
            ),
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = "[red]The config request has failed. Please check your connection and try again."
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        elif isinstance(e, requests.exceptions.ConnectionError):
            error_message += (
                " This could be due to maximum retries being exceeded or failure to establish a new connection. "
                "Please check your network configuration."
            )
        print(Panel(error_message + "[/red]"))

        # Print the traceback
        traceback_info = traceback.format_exc()
        print(traceback_info)

        raise typer.Exit(1)

    if not response.ok:
        print(Panel(f"[red]Sending config info to luma FAILED[/red]"))

    print_response(response)
    return response


def perform_request(request_info: RequestInfo) -> requests.Response:
    """Send an HTTP request based on the provided request information.

    Args:
        request_info (RequestInfo):

    """
    print(Panel(f"[yellow]Sending request to luma[/yellow]"))
    try:
        response = requests.request(
            method=request_info.method,
            url=request_info.url,
            headers=request_info.headers,
            params=request_info.params,
            json=request_info.payload,
            verify=request_info.verify,
            timeout=request_info.timeout,
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_messages = {
            requests.exceptions.Timeout: " If you're using a VPN, ensure it's properly connected or try disabling it temporarily.",
            requests.exceptions.ConnectionError: " This could be due to maximum retries being exceeded or failure to establish a new connection. Please check your network configuration.",
        }
        print(
            Panel(
                f"[red]The request has failed. Please check your connection and try again. {error_messages.get(type(e), '')}[/red]"
            )
        )
        # Print the traceback
        traceback_info = traceback.format_exc()
        print(traceback_info)

        raise typer.Exit(1)

    print_response(response)
    return response


def perform_ingestion_request(
    request_info: RequestInfo
) -> Tuple[requests.Response, Optional[str]]:
    """Send an HTTP request based on the provided request information and return the
        response and ingestion ID.

    Args:
        request_info (RequestInfo): Information about the request.

    Returns:
        Tuple[requests.Response, Optional[str]]: The HTTP response and the ingestion ID.
    """
    print(Panel(f"[yellow]Sending request to Luma[/yellow]"))
    ingestion_uuid = None
    try:
        response = requests.request(
            method=request_info.method,
            url=request_info.url,
            headers=request_info.headers,
            params=request_info.params,
            json=request_info.payload,
            verify=request_info.verify,
            timeout=request_info.timeout,
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = (
            "The request has failed. Please check your connection and try again."
        )
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        else:
            error_message += " This could be due to maximum retries being exceeded or failure to establish a new connection. Please check your network configuration."
        print(Panel(f"[red]{error_message}[/red]"))

        # Print the traceback
        traceback_info = traceback.format_exc()
        print(traceback_info)

        raise typer.Exit(1)

    if response.ok:
        success_message = "[green]The request was successful![/green]"
        print(Panel(success_message))
        try:
            response_json = response.json()
            # Extract ingestion ID from the response
            ingestion_uuid = response_json.get("ingestion_uuid")
            if ingestion_uuid:
                print(Panel(f"[green]Ingestion ID: {ingestion_uuid}[/green]"))
            else:
                print(
                    Panel("[yellow]No ingestion ID received in the response.[/yellow]")
                )
        except Exception as e:
            print(Panel(f"[red]Error processing response: {str(e)}[/red]"))
    else:
        try:
            error_message = f"URL: {response.url}\nAn HTTP error occurred, response status code: {response.status_code}\n{response.json().get('message', '')}"
        except Exception as e:
            error_message = f"URL: {response.url}\nAn HTTP error occurred, response status code: {response.status_code}\n{response.text}"
        print(Panel(f"[red]{error_message}[/red]"))

    return response, ingestion_uuid


def check_ingestion_status(luma_url: str, ingestion_uuid: str) -> str:
    """Fetches the status for a specific ingestion ID from Luma.

    Args:
        luma_url (str): The base URL for Luma.
        ingestion_uuid (str): The ingestion ID to fetch the status for.

    Returns:
        str: The status of the ingestion process.
    """
    status_endpoint = f"{luma_url}/api/v1/ingestions/"
    response = requests.get(
        status_endpoint, params={"uuid": ingestion_uuid}, verify=False
    )
    if not response.ok:
        print(
            f"Failed to fetch results for ingestion ID {ingestion_uuid}. HTTP Status: {response.status_code}"
        )
        raise typer.Exit(1)

    response_json = response.json()
    instance = response_json.get("data")
    status = instance.get("status")
    return status


def check_ingestion_results(luma_url: str, ingestion_uuid: str) -> Union[str, dict]:
    """Fetches and interprets the results for a specific ingestion ID.

    Args:
        luma_url (str): The base URL for Luma.
        ingestion_uuid (str): The ingestion ID to check the results for.

    Returns:
        Union[str, dict]: A message describing the status of the ingestion process or
            the JSON response for successful completions.
    """
    status_endpoint = f"{luma_url}/api/v1/ingestions/"
    response = requests.get(
        status_endpoint, params={"uuid": ingestion_uuid}, verify=False
    )
    if not response.ok:
        return f"Failed to fetch results for ingestion ID {ingestion_uuid}. HTTP Status: {response.status_code}"

    response_json = response.json()
    instance = response_json.get("data")
    status = instance.get("status")

    if status == IngestionStatus.pending.value:
        return f"Ingestion ID {ingestion_uuid} is still pending."
    elif status == IngestionStatus.failed.value:
        error_details = instance.get("error", "No additional error details provided.")
        return (
            f"Ingestion ID {ingestion_uuid} has failed. Error details: {error_details}"
        )
    elif status == IngestionStatus.successful.value:
        # Return the entire JSON response for successful completions
        return instance.get("summary")
    else:
        return f"Unrecognized status for ingestion ID {ingestion_uuid}: {status}"



"""Common Typer options."""

from pathlib import Path
from typing import Annotated

from rich import print
from rich.panel import Panel
import typer


def get_metadata_directory(metadata_dir: str | None, ctx: typer.Context) -> Path:
    """Return the current working directory if 'metadata_dir' is not specified.

    If 'metadata_dir' is provided, it returns the specified directory.

    Args:
        metadata_dir (str | None): Path to the metadata directory.
        ctx (typer.Context): Context of the current Typer command execution.

    Returns:
        Path: The current working directory or the specified metadata directory.
    """
    # Fix CLI completion; see
    # https://typer.tiangolo.com/tutorial/options/callback-and-context/#fix-completion-using-the-context.
    if ctx.resilient_parsing:
        return None

    if metadata_dir is not None:
        return metadata_dir

    cwd = Path.cwd()
    print(
        Panel(
            f"[yellow]'metadata_dir' not specified, using current working directory {cwd}[/yellow]"
        )
    )
    return cwd


MetadataDir: Annotated[
    str,
    typer.Option(
        "--metadata-dir",
        "-m",
        help="Specify the directory with dbt metadata files.",
        exists=True,
        dir_okay=True,
        resolve_path=True,
    ),
] = Path.cwd()

ConfigDir: Path = typer.Option(
    "./.luma",
    "--config-dir",
    "-c",
    help="Specify the directory with the config files. Defaults to ./.luma",
    envvar="LUMA_CONFIG_DIR",
    dir_okay=True,
    resolve_path=True,
)

Force: bool = typer.Option(
    False,
    "--force",
    "-f",
    help="Force the operation.",
)

DryRun: bool = typer.Option(
    False,
    "--dry-run",
    "-D",
    is_flag=True,
    help="Perform a dry run. Print the payload but do not send it.",
)

NoConfig: bool = typer.Option(
    False,
    "--no-config",
    "-n",
    help="Set this flag to prevent sending configuration data along with the request.",
)

Follow: bool = typer.Option(
    False, "--follow", help="Follow the ingestion process until it's completed."
)

IngestionId = Annotated[str, typer.Argument(help="Ingestion ID.")]

LumaURL: str = typer.Option(
    "localhost:8000",
    "--luma-url",
    "-l",
    help="URL of the luma instance.",
    envvar="LUMA_URL",
)

FollowTimeout: int = typer.Option(
    30,
    "--follow-timeout",
    "-t",
    help="How many seconds to wait for the ingestion process to complete.",
    envvar="LUMA_TIMEOUT",
)



from pathlib import Path

from luma.luma import app
from typer.testing import CliRunner

runner = CliRunner()


def test_ingest_requires_only_manifest_and_catalog_json(test_server, METADATA_DIR_V1_3):
    """Test if the 'ingest' command requires only manifest.json and catalog.json."""
    # Get result when the required files are present
    result_no_changes = get_result(
        test_server, "ingest", metadata_dir=METADATA_DIR_V1_3
    )

    # Get result when the rest of the json files are missing
    result_manifest_missing = get_result_after_renaming(
        test_server, "ingest", "manifest.json", metadata_dir=METADATA_DIR_V1_3
    )
    result_catalog_missing = get_result_after_renaming(
        test_server, "ingest", "catalog.json", metadata_dir=METADATA_DIR_V1_3
    )
    result_sources_missing = get_result_after_renaming(
        test_server, "ingest", "sources.json", metadata_dir=METADATA_DIR_V1_3
    )
    result_run_results_missing = get_result_after_renaming(
        test_server, "ingest", "run_results.json", metadata_dir=METADATA_DIR_V1_3
    )

    # Check ingest works when no files are changed, or sources.json or run_results.json
    # are missing.
    assert result_no_changes.exit_code == 0
    assert result_sources_missing.exit_code == 0
    assert result_run_results_missing.exit_code == 0

    # Check ingest fails when either manifest.json or catalog.json is missing
    assert result_manifest_missing.exit_code == 1
    assert result_catalog_missing.exit_code == 1


def test_sendtestresults_requires_only_run_results_json(test_server, METADATA_DIR_V1_3):
    """Test if the 'send-test-results' command requires only run_results.json."""
    # Get result when the required files are present
    result_no_changes = get_result(
        test_server, "send-test-results", metadata_dir=METADATA_DIR_V1_3
    )

    # Get result when the rest of the json files are missing
    result_manifest_missing = get_result_after_renaming(
        test_server,
        "send-test-results",
        "manifest.json",
        metadata_dir=METADATA_DIR_V1_3,
    )
    result_catalog_missing = get_result_after_renaming(
        test_server, "send-test-results", "catalog.json", metadata_dir=METADATA_DIR_V1_3
    )
    result_sources_missing = get_result_after_renaming(
        test_server, "send-test-results", "sources.json", metadata_dir=METADATA_DIR_V1_3
    )
    result_run_results_missing = get_result_after_renaming(
        test_server,
        "send-test-results",
        "run_results.json",
        metadata_dir=METADATA_DIR_V1_3,
    )

    # Check ingest works when no files are changed, or any other json file but
    # run_result.json is missing.
    assert result_no_changes.exit_code == 0
    assert result_sources_missing.exit_code == 0
    assert result_manifest_missing.exit_code == 0
    assert result_catalog_missing.exit_code == 0

    # Check ingest fails when run_results.json is missing
    assert result_run_results_missing.exit_code == 1


# Rename files, get result, and rename them back
def get_result_after_renaming(test_server, command, file_name, metadata_dir):
    Path.rename(f"{metadata_dir}/{file_name}", f"{metadata_dir}/missing_{file_name}")
    result = get_result(test_server, command, metadata_dir=metadata_dir)
    Path.rename(f"{metadata_dir}/missing_{file_name}", f"{metadata_dir}/{file_name}")
    return result


# Get result from invoking the runner ("ingest" is the default command)
def get_result(test_server, command, metadata_dir):
    return runner.invoke(
        app,
        [
            "dbt",
            command,
            "--metadata-dir",
            metadata_dir,
            "--luma-url",
            test_server,
        ],
    )



from typing import Generator

from loguru import logger

from luma.metadata.models.database import DatabaseTableManifest
from luma.metadata.sources.sap.extract import sap
from luma.metadata.sources.sap.transform import transform


def pipeline() -> Generator[DatabaseTableManifest, None, None]:
    """Pipeline to extract SAP metadata and."""
    source = sap().with_resources("column_details", "custom_tables_details")
    manifest_batches = transform()
    yield from manifest_batches


if __name__ == "__main__":
    manifest_batches = pipeline()
    for i, manifest in enumerate(manifest_batches):
        logger.info(f"Writing {len(manifest.payload)} tables to batch {i} manifest...")

        with open(f"database_table_manifest__batch_{i}.json", "w") as f:
            f.write(manifest.json())



from pathlib import Path
from typing import Dict, List

import typer
from rich import print
from rich.panel import Panel

from lumaCLI.models import Config, RequestInfo
from lumaCLI.utils import get_config, get_db_metadata, send_config, perform_request
from lumaCLI.utils.options import (
    ConfigDir,
    DryRun,
    LumaURL,
    NoConfig,
    PostgresDatabase,
    PostgresHost,
    PostgresPassword,
    PostgresPort,
    PostgresUsername,
)

app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)


@app.command()
def ingest(
    luma_url: str = LumaURL,
    username: str = PostgresUsername,
    database: str = PostgresDatabase,
    host: str = PostgresHost,
    port: str = PostgresPort,
    password: str = PostgresPassword,
    dry_run: bool = DryRun,
    config_dir: Path = ConfigDir,
    no_config: bool = NoConfig,
) -> RequestInfo:
    """
    Ingests metadata from a PostgreSQL database and sends it to a specified Luma ingestion endpoint.
    """

    should_send_config = not no_config
    config = None
    # get_config

    if should_send_config:
        try:
            config: Config = get_config(config_dir=config_dir)
        except FileNotFoundError:
            print(
                Panel(
                    f"[blue]No config files found. Continuing with the operation...[/blue]"
                )
            )

    # Retrieve database metadata
    db_metadata: Dict[str, List[Dict]] = get_db_metadata(
        username=username, database=database, host=host, port=port, password=password
    )

    # In dry run mode, print the database metadata and exit
    if dry_run:
        print(db_metadata)
        raise typer.Exit(0)

    endpoint = f"{luma_url}/api/v1/postgres"

    # Create the request info object to return
    request_info = RequestInfo(
        url=endpoint,
        method="POST",
        payload=db_metadata,
        verify=False,
    )
    if config and should_send_config:
        config_response = send_config(config=config, luma_url=luma_url)

    response = perform_request(request_info)
    if not response.ok:
        raise typer.Exit(1)

    return response


if __name__ == "__main__":
    app()



"""Utility functions for the CLI."""

import contextlib
from enum import Enum
import json
from pathlib import Path
import subprocess
import traceback
from typing import Optional, Union

import requests
from rich import print
from rich.console import Console
from rich.panel import Panel
import typer
import yaml

from luma.models import Config


class IngestionStatus(Enum):
    successful = 0
    failed = 1
    pending = 2


class HttpMethod(str, Enum):
    """HTTP methods."""

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    PATCH = "PATCH"


# Create console for rich output
console = Console()

CONFIG_YAML_EXAMPLE = """# Example:
#
# groups:
#   - meta_key: "domain"
#     slug: "domains"
#     label_plural: "Domains"
#     label_singular: "Domain"
#     icon: "Cube"
#     in_sidebar: true
#     visible: true
#   - meta_key: "true_source"
#     slug: "sources"
#     label_plural: "Sources"
#     label_singular: "Source"
#     icon: "Cloud"
#     in_sidebar: true
"""
OWNERS_YAML_EXAMPLE = """# Example:
#
# owners:
#   - email: "some@one.com"
#     first_name: "Dave"
#     last_name: "Smith"
#     title: "Director"
#   - email: "other@person.com"
#     first_name: "Michelle"
#     last_name: "Dunne"
#     title: "CTO"
#   - email: "someone@else.com"
#     first_name: "Dana"
#     last_name: "Pawlak"
#     title: "HR Manager"
"""


def json_to_dict(json_path: str) -> dict | None:
    """Convert a JSON file to a Python dictionary.

    Args:
        json_path (str): The path to the JSON file.

    Returns:
        optional[dict]: A dictionary representation of the JSON file, or None if an
            error occurs.
    """
    try:
        with Path.open(json_path, "r") as json_file:
            data = json.load(json_file)
    except Exception:
        data = None
    return data


def run_command(command: str, capture_output: bool = False) -> str | None:
    """Execute a shell command and optionally capture its output.

    Args:
        command (str): The shell command to be executed.
        capture_output (bool, optional): Flag to determine if the command's output
            should be captured. Defaults to False.

    Returns:
        optional[str]: The standard output of the command if `capture_output` is True,
            otherwise None.

    Raises:
        typer.Exit: Exits the script if the command execution fails.
    """
    try:
        result = subprocess.run(
            command,
            shell=True,  # noqa: S602
            check=True,
            capture_output=True,
            text=True,
        )
    except subprocess.CalledProcessError as e:
        console.print(
            Panel.fit(
                f"[bold red]ERROR[/bold red]: An error occurred while running the command: [bold yellow]{e}[/bold yellow]",
                title="Error",
                border_style="red",
            )
        )
        if e.output:
            console.print(f"[bold cyan]Output[/bold cyan]: {e.output}")
        if e.stderr:
            console.print(f"[bold red]Error[/bold red]: {e.stderr}")
        raise typer.Exit(1) from e

    if capture_output:
        return result.stdout.strip()

    return None


def init_config(config_dir: Path | str = "./.luma", force: bool = False) -> None:
    """Initialize configuration files in the specified directory.

    Args:
        config_dir (Path | str, optional): The directory where configuration files
            will be created. Defaults to "./.luma".
        force (bool, optional): If True, existing configuration files will be
            overwritten. Defaults to False.

    Raises:
        FileExistsError: If configuration files already exist and `force` is not set to
            True.
    """
    config_dir = Path(config_dir)

    config_path = config_dir / "config.yaml"
    owners_path = config_dir / "owners.yaml"

    if force:
        config_path.unlink(missing_ok=True)
        owners_path.unlink(missing_ok=True)
        with contextlib.suppress(FileNotFoundError):
            config_dir.rmdir()

    if not config_path.exists() and not owners_path.exists():
        config_dir.mkdir(exist_ok=True)
        config_path.touch(exist_ok=False)
        owners_path.touch(exist_ok=False)
    else:
        raise FileExistsError

    config_path.write_text(CONFIG_YAML_EXAMPLE)
    owners_path.write_text(OWNERS_YAML_EXAMPLE)


def get_config(config_dir: Path | str = "./.luma") -> Config | None:
    """Retrieve configuration data from YAML files in the specified directory.

    Args:
        config_dir (Path | str, optional): The directory containing the
            configuration files. Defaults to "./.luma".

    Returns:
        optional[Config]: The configuration object if the configuration is successfully
            loaded, otherwise None.

    Raises:
        FileNotFoundError: If the configuration files are missing.
        typer.Abort: If there is an error parsing the YAML files.
    """
    config_dir = Path(config_dir)

    config_path = config_dir / "config.yaml"
    owners_path = config_dir / "owners.yaml"

    config_missing = True
    owners_missing = True

    config_dict = {}
    config_data = {}
    owners_data = {}

    if config_path.exists():
        config_missing = False
        with config_path.open("r") as f:
            try:
                config_data: Optional[dict] = yaml.safe_load(f)
            except yaml.YAMLError as e:
                print(f"Error parsing YAML file: {e}")
                raise typer.Abort() from e

    if owners_path.exists():
        owners_missing = False
        with owners_path.open("r") as f:
            try:
                owners_data: Optional[dict] = yaml.safe_load(f)

            except yaml.YAMLError as e:
                print(f"Error parsing YAML file: {e}")
                raise typer.Abort() from e

    if config_missing and owners_missing:
        raise FileNotFoundError

    if config_data is not None:
        config_dict.update(config_data)

    if owners_data is not None:
        config_dict.update(owners_data)

    return Config(**config_dict)


def print_response(response: requests.Response) -> None:
    """Print Luma API response.

    Args:
        response (requests.Response): The response to print.
    """
    try:
        parsed_response = response.json()
    except Exception:
        parsed_response = None

    if response.ok:
        msg = "[green]The request was successful.[/green]"
        if parsed_response:
            msg += f"\n[yellow]Response:\n{parsed_response}[/yellow]"
        else:
            msg += f"\n[green]Raw response:\n{response.text}[/green]"
    else:
        if parsed_response:
            error_msg = f"\n\nError message: {parsed_response['message']}"
        else:
            error_msg = f"Raw response:\n{response.text}"
        msg = f"[red]The request failed with status code {response.status_code}.{error_msg}[/red]"

    print(msg)

    if not response.ok or parsed_response is None:
        raise typer.Exit(1)


def send_config(
    config: Config, luma_url: str, verify: bool = True
) -> requests.Response:
    """Send configuration data to a specified URL.

    Args:
        config (Config): The configuration data to be sent.
        luma_url (str): The URL where the configuration data will be sent.
        verify (bool, optional): Whether to verify the server's TLS certificate.

    Returns:
        requests.Response: The response from the server after sending the configuration
            data.

    Raises:
        typer.Exit: If there is an error in sending the configuration data.
    """
    print(Panel("[yellow]Sending config info to luma[/yellow]"))

    try:
        response = requests.request(
            method="POST",
            url=f"{luma_url}/api/v1/config",
            json=config.dict(),
            verify=verify,
            timeout=(
                21.05,
                60 * 30,
            ),
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = "[red]The config request has failed. Please check your connection and try again."
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        elif isinstance(e, requests.exceptions.ConnectionError):
            error_message += (
                " This could be due to maximum retries being exceeded or failure to establish a new connection. "
                "Please check your network configuration."
            )
        print(Panel(error_message + "[/red]"))

        # Print the traceback
        traceback_info = traceback.format_exc()
        print(traceback_info)

        raise typer.Exit(1) from e

    if not response.ok:
        print(Panel("[red]Sending config info to luma FAILED[/red]"))

    print_response(response)
    return response


def perform_ingestion_request(
    url: str,
    payload: dict | list | None,
    verify: bool = True,
    method: HttpMethod = HttpMethod.POST,
    timeout: (float | tuple[float, float] | tuple[float, None]) | None = (
        20,
        60 * 30,
    ),
    headers: dict[str, str] | None = None,
    params: dict[str, str | int | float] | None = None,
) -> tuple[requests.Response, str | None]:
    """Send an HTTP request.

    Args:
        url (str): The URL for the request.
        payload (dict | list | None): The payload for the request, if any.
        verify (bool, optional): Whether to verify the server's TLS certificate.
            Defaults to True.
        method (HttpMethod, optional): _description_. Defaults to HttpMethod.POST.
        timeout (optional[float | tuple[float, float] | tuple[float, None]]): The
            timeout for the request. Defaults to (20, 60 * 30).
        headers (optional[dict[str, str]], optional): Headers to be sent with the
            request. Defaults to None.
        params (optional[dict[str, str | int | float]]): URL parameters for the request.
            Defaults to None.

    Raises:
        typer.Exit: In case of timeout or connection error.

    Returns:
        tuple[requests.Response, str | None]: The HTTP response and the ingestion ID.
    """
    print("[yellow]Sending ingestion request to Luma...[/yellow]")

    try:
        response = requests.request(
            method=method,
            url=url,
            headers=headers,
            params=params,
            json=payload,
            verify=verify,
            timeout=timeout,
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = (
            "The request has failed. Please check your connection and try again."
        )
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        else:
            error_message += " This could be due to maximum retries being exceeded or failure to establish a new connection. Please check your network configuration."
        print(Panel(f"[red]{error_message}[/red]"))

        # Print the traceback
        traceback_info = traceback.format_exc()
        print(traceback_info)

        raise typer.Exit(1) from e

    print_response(response)

    ingestion_id = response.json().get("ingestion_uuid")
    return response, ingestion_id


def check_ingestion_status(
    luma_url: str, ingestion_uuid: str, verify: bool = True
) -> str:
    """Fetches the status for a specific ingestion ID from Luma.

    Args:
        luma_url (str): The base URL for Luma.
        ingestion_uuid (str): The ingestion ID to fetch the status for.
        verify (bool, optional): Whether to verify the server's TLS certificate.
            Defaults to True.

    Returns:
        str: The status of the ingestion process.
    """
    status_endpoint = f"{luma_url}/api/v1/ingestions/"
    response = requests.get(
        status_endpoint,
        params={"uuid": ingestion_uuid},
        verify=verify,
        timeout=(20, 60 * 30),
    )
    if not response.ok:
        print(
            f"Failed to fetch results for ingestion ID {ingestion_uuid}. HTTP Status: {response.status_code}"
        )
        raise typer.Exit(1)

    response_json = response.json()
    instance = response_json.get("data")
    return instance.get("status")


def check_ingestion_results(
    luma_url: str, ingestion_uuid: str, verify: bool = True
) -> Union[str, dict]:
    """Fetches and interprets the results for a specific ingestion ID.

    Args:
        luma_url (str): The base URL for Luma.
        ingestion_uuid (str): The ingestion ID to check the results for.
        verify (bool, optional): Whether to verify the server's TLS certificate.
            Defaults to True.

    Returns:
        Union[str, dict]: A message describing the status of the ingestion process or
            the JSON response for successful completions.
    """
    status_endpoint = f"{luma_url}/api/v1/ingestions/"
    response = requests.get(
        status_endpoint,
        params={"uuid": ingestion_uuid},
        verify=verify,
        timeout=(20, 60 * 30),
    )
    if not response.ok:
        return f"Failed to fetch results for ingestion ID {ingestion_uuid}. HTTP Status: {response.status_code}"

    response_json = response.json()
    instance = response_json.get("data")
    status = instance.get("status")

    if status == IngestionStatus.pending.value:
        return f"Ingestion ID {ingestion_uuid} is still pending."

    if status == IngestionStatus.failed.value:
        error_details = instance.get("error", "No additional error details provided.")
        return (
            f"Ingestion ID {ingestion_uuid} has failed. Error details: {error_details}"
        )

    if status == IngestionStatus.successful.value:
        # Return the entire JSON response for successful completions
        return instance.get("summary")

    return f"Unrecognized status for ingestion ID {ingestion_uuid}: {status}"



import os
from importlib.metadata import version
from pathlib import Path

import typer
from rich import print
from rich.panel import Panel
from typing_extensions import Annotated

CLI_NAME = "luma"
__version__ = version("lumaCLI")


# Utility functions
def get_current_working_directory(metadata_dir, ctx: typer.Context) -> Path:
    """
    Returns the current working directory if 'metadata_dir' is not specified.
    If 'metadata_dir' is provided, it returns the specified directory.

    Args:
    - metadata_dir: Path to the metadata directory.
    - ctx: Context of the current Typer command execution.

    Returns:
    - Path: The current working directory or the specified metadata directory.
    """
    # Returns current working directory if metadata_dir is not specified
    if ctx.resilient_parsing:
        return
    if metadata_dir is not None:
        return metadata_dir
    cwd = Path(os.getcwd())
    print(
        Panel(
            f"[yellow]'metadata_dir' not specified, using current working directory {cwd}[/yellow]"
        )
    )
    return cwd


# Callback functions
def require_if_not_dry_run(value, param: typer.CallbackParam, ctx: typer.Context):
    """
    Ensures that the given value is required if the 'dry_run' flag is not set.

    Args:
    - value: The value to be checked.
    - param: The parameter associated with the value.
    - ctx: Context of the current Typer command execution.

    Returns:
    - str: The validated value if the conditions are met.

    Raises:
    - typer.BadParameter: If the value is missing and 'dry_run' is not enabled.
    """
    if ctx.resilient_parsing:
        return
    dry_run = ctx.params.get("dry_run", False)

    if value or dry_run:
        return value

    # Raise a typer.BadParameter exception to simulate the same error as missing 'endpoint'
    raise typer.BadParameter(f"Missing argument '{param.name.upper()}'")


# Command Line Options

MetadataDir: Path = typer.Option(
    None,
    "--metadata-dir",
    "-m",
    help="Specify the directory with dbt metadata files. Defaults to current working directory if not provided.",
    callback=get_current_working_directory,
    exists=True,
    dir_okay=True,
    resolve_path=True,
)
ConfigDir: Path = typer.Option(
    "./.luma",
    "--config-dir",
    "-c",
    help="Specify the directory with the config files. Defaults to ./.luma",
    envvar="LUMA_CONFIG_DIR",
    dir_okay=True,
    resolve_path=True,
)
Force: bool = typer.Option(
    False,
    "--force",
    "-f",
    help="Force the operation.",
)

DryRun: bool = typer.Option(
    False,
    "--dry-run",
    "-D",
    is_flag=True,
    help="Perform a dry run. Print the payload but do not send it.",
)

NoConfig: bool = typer.Option(
    False,
    "--no-config",
    "-n",
    help="Set this flag to prevent sending configuration data along with the request.",
)

Follow: bool = typer.Option(
    False, "--follow", help="Follow the ingestion process until it's completed."
)

IngestionId = Annotated[str, typer.Argument(help="Ingestion UUID.")]

LumaURL: str = typer.Option(
    None,
    "--luma-url",
    "-l",
    help="URL of the luma instance.",
    envvar="LUMA_URL",
    callback=require_if_not_dry_run,
)
PostgresUsername: str = typer.Option(
    ...,
    "--username",
    "-u",
    envvar="LUMA_POSTGRES_USERNAME",
    help="The username for the PostgreSQL database.",
    prompt="PostgreSQL username",
)
PostgresDatabase: str = typer.Option(
    ...,
    "--database",
    "-d",
    envvar="LUMA_POSTGRES_DATABASE",
    help="The name of the PostgreSQL database.",
    prompt="PostgreSQL database",
)
PostgresHost: str = typer.Option(
    "localhost",
    "--host",
    "-h",
    envvar="LUMA_POSTGRES_HOST",
    help="The host address of the PostgreSQL database.",
)
PostgresPort: str = typer.Option(
    "5432",
    "--port",
    "-p",
    envvar="LUMA_POSTGRES_PORT",
    help="The port number for the PostgreSQL database.",
)
PostgresPassword: str = typer.Option(
    ...,
    "--password",
    "-P",
    envvar="LUMA_POSTGRES_PASSWORD",
    help="The password for the PostgreSQL database.",
    prompt="PostgreSQL password",
    hide_input=True,
)



"""Utility functions."""

from luma.utils.luma_utils import (
    check_ingestion_results,
    check_ingestion_status,
    get_config,
    init_config,
    json_to_dict,
    perform_ingestion_request,
    print_response,
    run_command,
    send_config,
)
from luma.utils.postgres_utils import (
    create_conn,
    generate_pg_dump_content,
    get_db_metadata,
    get_pg_dump_tables_info,
    get_pg_dump_views_info,
    get_tables_row_counts,
    get_tables_size_info,
)
from luma.utils.state import state


__all__ = [
    "check_ingestion_results",
    "check_ingestion_status",
    "create_conn",
    "generate_pg_dump_content",
    "get_config",
    "get_db_metadata",
    "get_pg_dump_tables_info",
    "get_pg_dump_views_info",
    "get_tables_row_counts",
    "get_tables_size_info",
    "init_config",
    "json_to_dict",
    "perform_ingestion_request",
    "print_response",
    "run_command",
    "send_config",
    "state",
]



"""
This module implements a command-line interface (CLI) for the Luma application.
It provides commands for database operations, configuration management, and other utilities.
"""

import importlib.metadata
from typing import Union

import typer

# Standard library imports
import urllib3
from rich.panel import Panel

import lumaCLI.commands.config as config
import lumaCLI.commands.dbt as dbt
import lumaCLI.commands.postgres as postgres
from lumaCLI.utils import check_ingestion_results
from lumaCLI.utils.options import CLI_NAME, IngestionId, LumaURL

__version__ = importlib.metadata.version(__package__ or __name__)

# Disable warnings related to insecure requests for specific cases
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# Create a Typer application with configured properties
app = typer.Typer(
    name=CLI_NAME,
    no_args_is_help=True,
    pretty_exceptions_show_locals=False,
    pretty_exceptions_enable=True,
    pretty_exceptions_short=True,
)


def print_did_you_mean_luma():
    """Prints a suggestion panel if 'lumaCLI' is mistakenly used."""
    print(
        Panel(
            f"Whoops, you typed [bold red]lumaCLI[/bold red], did you mean '[bold green]luma[/bold green]' ??",
            border_style="blue",
        )
    )


def version_callback(show_version: bool):
    """
    Prints the version of the application and exits.

    Args:
        show_version (bool): If True, shows the version.
    """

    if show_version:
        typer.echo(__version__)
        raise typer.Exit()


@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        is_eager=True,
        help="Show the version and exit.",
    ),
):
    """
    Main function for the Typer application.

    Args:
        version (bool): Flag to show the version and exit.
    """
    pass


@app.command()
def status(
    ingestion_uuid: IngestionId,
    luma_url: str = LumaURL,
) -> Union[str, dict]:
    """Retrieve the status of an ingestion."""
    results = check_ingestion_results(luma_url, ingestion_uuid)
    print(f"Ingestion results for ID {ingestion_uuid}:")
    print(results)
    return results


# Add commands to the Typer application
app.add_typer(dbt.app, name="dbt", help="Ingest metadata from dbt.")
app.add_typer(postgres.app, name="postgres", help="Ingest metadata from Postgres.")
app.add_typer(config.app, name="config", help="Manage Luma instance configuration.")



"""Utility functions for working with PostgreSQL metadata."""

import os
import re

import psycopg
from rich.console import Console

from luma.utils import run_command


console = Console()


def create_conn(
    username: str, password: str, host: str, port: str, database: str
) -> psycopg.Connection:
    """Create and return a new database connection using the given credentials.

    Args:
        username (str): Username for the database.
        password (str): Password for the database.
        host (str): Host address of the database.
        port (str): Port number for the database connection.
        database (str): Name of the database to connect to.

    Returns:
        psycopg.extensions.connection: A new database connection.
    """
    return psycopg.connect(
        user=username, password=password, host=host, port=port, dbname=database
    )


def generate_pg_dump_content(
    username: str,
    database: str,
    password: str,
    host: str = "localhost",
    port: str = "5432",
) -> str:
    """Generate a string dump of the PostgreSQL database schema, tables, etc.

    Args:
        username (str): The username for the database.
        database (str): The name of the database.
        password (str): The password for the database.
        host (str, optional): The host of the database. Defaults to "localhost".
        port (str, optional): The port of the database. Defaults to "5432".

    Returns:
        str: The result of the pg_dump command as a string.
    """
    original_pgpassword_env = os.getenv("PGPASSWORD")
    os.environ["PGPASSWORD"] = password
    command = f"pg_dump -h {host} -p {port} -U {username} -d {database} --column-inserts --rows-per-insert=1 --no-password"
    result = run_command(command, capture_output=True)
    os.environ["PGPASSWORD"] = original_pgpassword_env
    return result


def get_pg_dump_tables_info(content: str) -> list[dict]:
    """Extract table information from a PostgreSQL database dump content.

    Args:
        content (str): PostgreSQL database dump content as a string.

    Returns:
        list[dict]: A list of dictionaries, each containing information about a table
            in the database.
    """
    # Find table creation statements
    table_creations = re.findall(
        r"CREATE TABLE (.*?)\.(.*?) \((.*?)\);", content, re.DOTALL
    )

    # Find row counts and table sizes
    row_counts = re.findall(
        r"-- Name: (.*?); Type: TABLE;.*?-- Total rows: (\base_dict+)", content
    )
    table_sizes = re.findall(
        r"-- Name: (.*?); Type: TABLE;.*?-- Size: (.*?)\n", content
    )

    # Find table owners
    table_owners = re.findall(r"ALTER TABLE (.*?)\.(.*?) OWNER TO (.*?);", content)

    # Find column defaults
    column_defaults = re.findall(
        r"ALTER TABLE ONLY (.*?)\.(.*?) ALTER COLUMN (.*?) SET DEFAULT (.*?);", content
    )

    # Find primary keys
    primary_keys = re.findall(
        r"ALTER TABLE ONLY (.*?)\.(.*?)\s+ADD CONSTRAINT (.*?) PRIMARY KEY \((.*?)\);",
        content,
    )

    # Find foreign keys
    foreign_keys = re.findall(
        r"ALTER TABLE ONLY (.*?)\.(.*?)\s+ADD CONSTRAINT (.*?) FOREIGN KEY \((.*?)\) REFERENCES (.*?)\.(.*?)\(id\);",
        content,
    )

    # Initialize the dictionary
    tables_list = []

    # Process each table
    for table_schema, table_name, table_definition in table_creations:
        # Extract column names and types
        columns = re.findall(r"(\w+)\s+([\w\s()]+)(?:,|\))", table_definition)

        # Store column names and types in a dictionary
        columns_dict = {
            column_name: column_type.strip() for column_name, column_type in columns
        }

        # Find row count for the table
        row_count = next((count for name, count in row_counts if name == table_name), 0)

        # Find table size (in MB) for the table
        table_size = next(
            (size for name, size in table_sizes if name == table_name), "Unknown"
        )

        # Find table owner
        table_owner = next(
            (owner for schema, name, owner in table_owners if name == table_name),
            "Unknown",
        )

        # Find column defaults
        column_default_dict = {
            column_name: default_value
            for schema, table, column_name, default_value in column_defaults
            if table == table_name
        }

        # Find primary key
        primary_key = next(
            (key for schema, name, _, key in primary_keys if name == table_name), None
        )

        # Find foreign keys
        foreign_keys_list = [
            {"constraint_name": name, "column": column, "ref_table": ref_table}
            for schema, table, name, column, ref_schema, ref_table in foreign_keys
            if table == table_name
        ]

        # Add the table details to the schema dictionary
        tables_list.append({
            "table_name": table_name,
            "table_schema": table_schema,
            "columns": columns_dict,
            "row_count": int(row_count),
            "table_size": table_size,
            "owner": table_owner,
            "column_defaults": column_default_dict,
            "primary_key": primary_key,
            "foreign_keys": foreign_keys_list,
        })

    return tables_list


def get_pg_dump_views_info(content: str) -> list[dict]:
    """Extract view information from a PostgreSQL database dump content.

    Args:
        content (str): PostgreSQL database dump content as a string.

    Returns:
        list[dict]: A list of dictionaries, each containing information about a view in
            the database.
    """
    # Find view creation statements
    view_creations = re.findall(
        r"CREATE VIEW (.*?)\.(.*?) AS(.*?);", content, re.DOTALL
    )

    # Initialize the list for storing view information
    views_list = []

    # Process each view
    for view_schema, view_name, view_definition in view_creations:
        # Remove newline characters and extra spaces
        view_definition_clean = re.sub(r"\s+", " ", view_definition)

        # Extract column names and aliases
        columns = re.findall(r"(\w+)\s+AS\s+(\w+)", view_definition_clean)

        # Add the view details to the list
        views_list.append({
            "view_name": view_name,
            "view_schema": view_schema,
            "columns": dict(columns),
        })

    return views_list


def get_tables_size_info(
    username: str,
    database: str,
    password: str,
    host: str = "localhost",
    port: str = "5432",
) -> dict:
    """Retrieve size information for all tables in the specified PostgreSQL database.

    Args:
        username (str): Database username.
        database (str): Database name.
        password (str): Database password.
        host (str, optional): Database host. Defaults to "localhost".
        port (str, optional): Database port. Defaults to "5432".

    Returns:
        dict: A dictionary where keys are table names and values are their respective
            sizes.
    """
    conn = create_conn(username, password, host, port, database)
    cur = conn.cursor()
    cur.execute(
        """
        SELECT table_schema,
               table_name,
               pg_size_pretty(pg_total_relation_size(('"' || table_schema || '"."' || table_name || '"'))) AS table_size
        FROM information_schema.tables
        ORDER BY pg_total_relation_size(('"' || table_schema || '"."' || table_name || '"')) DESC;
    """
    )
    rows = cur.fetchall()

    table_size_dict = {}

    for table_schema, table_name, table_size in rows:
        table_size_dict[(table_schema, table_name)] = table_size
    return table_size_dict


def get_tables_row_counts(
    username: str,
    database: str,
    password: str,
    host: str = "localhost",
    port: str = "5432",
) -> dict:
    """Retrieve row counts for all tables in a specified PostgreSQL database.

    Args:
        username (str): Database username.
        database (str): Database name.
        password (str): Database password.
        host (str, optional): Database host. Defaults to "localhost".
        port (str, optional): Database port. Defaults to "5432".

    Returns:
        dict: A dictionary where keys are table names and values are the number of rows
            in each table.
    """
    conn = create_conn(username, password, host, port, database)
    cur = conn.cursor()
    cur.execute(
        """
        SELECT schemaname,relname,n_live_tup
        FROM pg_stat_user_tables
        ORDER BY n_live_tup DESC;
    """
    )
    rows = cur.fetchall()
    row_count_dict = {}

    for table_schema, table_name, row_count in rows:
        row_count_dict[(table_schema, table_name)] = row_count
    return row_count_dict


def get_db_metadata(
    username: str,
    database: str,
    password: str,
    host: str = "localhost",
    port: str = "5432",
) -> dict[str, list[dict]]:
    """Gather metadata about the specified PostgreSQL database, tables, and views.

    Args:
        username (str): Database username.
        database (str): Database name.
        password (str): Database password.
        host (str, optional): Database host. Defaults to "localhost".
        port (str, optional): Database port. Defaults to "5432".

    Returns:
        dict[str, list[dict]]: A dictionary containing metadata about tables and views
            in the database.
    """
    dump_content: str = generate_pg_dump_content(
        username=username, database=database, host=host, port=port, password=password
    )
    # Transform SQL dump to dictionary
    tables_info: list[dict] = get_pg_dump_tables_info(dump_content)

    # Get row count information
    row_count_dict: dict = get_tables_row_counts(
        username=username, database=database, host=host, port=port, password=password
    )

    # Get table size information:
    table_size_dict: dict = get_tables_size_info(
        username=username, database=database, host=host, port=port, password=password
    )

    for table in tables_info:
        key = (table["table_schema"], table["table_name"])
        table["row_count"] = row_count_dict.get(key, 0)
        table["table_size"] = table_size_dict.get(key, 0)

    views_info: list[dict] = get_pg_dump_views_info(dump_content)

    return {"tables": tables_info, "views": views_info}


if __name__ == "__main__":
    pass



from enum import Enum
from typing import Dict, Optional, Tuple, Union

from pydantic import AnyHttpUrl, BaseModel, validator


class HttpMethod(str, Enum):
    """
    Enumeration for HTTP methods.
    """
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    PATCH = "PATCH"


class RequestInfo(BaseModel):
    """
    Information about an HTTP request.

    Attributes:
        url (AnyHttpUrl): The URL for the request.
        payload (Optional[Union[Dict, list]]): The payload for the request, if any.
        verify (bool): Whether to verify the server's TLS certificate.
        method (HttpMethod): The HTTP method for the request. Defaults to POST.
        timeout (Optional[Union[float, Tuple[float, float], Tuple[float, None]]]): The timeout for the request.
        headers (Optional[Dict[str, str]]): Headers to be sent with the request.
        params (Optional[Dict[str, Union[str, int, float]]]): URL parameters for the request.
    """
    url: AnyHttpUrl
    payload: Optional[Union[Dict, list]]
    verify: bool = True
    method: HttpMethod = HttpMethod.POST
    timeout: Optional[Union[float, Tuple[float, float], Tuple[float, None]]] = (
        20,
        60 * 30,
    )
    headers: Optional[Dict[str, str]] = None
    params: Optional[Dict[str, Union[str, int, float]]] = None

    @validator("headers", pre=True, always=True)
    def set_default_headers(cls, v):
        """
        Ensures that default headers are set if none are provided.

        Args:
            v: The value of the headers.

        Returns:
            The headers with a default if none were provided.
        """
        return v or {"Content-Type": "application/json"}

    @validator("method", pre=True)
    def uppercase_method(cls, v):
        """
        Ensures that the HTTP method is in uppercase.

        Args:
            v: The HTTP method.

        Returns:
            The HTTP method in uppercase.
        """
        return v.upper() if isinstance(v, str) else v



"""Manage Luma instance configuration."""

from pathlib import Path

from rich import print
import typer

from luma.utils import get_config, init_config, send_config
from luma.utils.options import ConfigDir, DryRun, Force, LumaURL


app = typer.Typer(
    name="config", no_args_is_help=True, pretty_exceptions_show_locals=False
)


@app.command(help="Initialize the configuration.")
def init(config_dir: Path = ConfigDir, force: bool = Force) -> None:
    """Initialize the configuration.

    Args:
        config_dir (Path): The directory to write the configuration files.
        force (bool): If True, overwrite the configuration if it already exists.

    Raises FileExistsError if the configuration already exists and 'force' is not True.
    """
    try:
        init_config(config_dir=config_dir, force=force)
        print(f"[green]Config initialized at[/green] {config_dir}")
    except FileExistsError as e:
        print(
            f"[red]Error![/red] [red]Config files already exist at[/red] {config_dir}\n"
            f"[yellow]If you want to override run with flag [/yellow][red]--force/-f[/red]"
        )
        raise typer.Exit(1) from e


@app.command(help="Display the current configuration information.")
def show(config_dir: Path = ConfigDir) -> None:
    """Display current configuration from the specified directory."""
    try:
        config = get_config(config_dir=config_dir)
        print(config)
    except FileNotFoundError as e:
        print(
            "[red]Error![/red] [red]Config files not found at[/red] {config_dir}\n"
            "[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
        )
        raise typer.Exit(1) from e


@app.command(help="Send the current configuration information to luma")
def send(
    config_dir: Path = ConfigDir, luma_url: str = LumaURL, dry_run: bool = DryRun
) -> None:
    """Send configuration to the specified Luma URL.

    In dry run mode, the configuration is printed but not sent.
    """
    try:
        config = get_config(config_dir=config_dir)

        if dry_run:
            print(config.dict())
            return

        if config:
            response = send_config(config=config, luma_url=luma_url)
            if not response.ok:
                raise typer.Exit(1)
        else:
            print(
                f"[red]No Config detected under {config_dir}[/red]\n"
                f"[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
            )

    except FileNotFoundError as e:
        print(
            f"[red]Error![/red] [red]Config files not found at[/red] {config_dir}\n"
            f"[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
        )
        raise typer.Exit(1) from e
